package de.hdm_stuttgart.sw1.klausur.angebot;

/**
 * Erstellung von Angeboten für Dienstleistungen eines Handwerksbetriebs. Erfaßt werden:
 * 
 * <ul>
 *   <li>Bodenverlegung in Räumen: Kosten für Boden und Fußleisten</li>
 *   <li>Berücksichtigung der Anzahl von Anfahrtswegen</li>
 * </ul>
 * 
 * <p>Im Hinblick auf den Kostenanteil für Anfahrtswege wird der Kilometerpreis über den Konstruktor
 * des jeweiligen Angebots festgelegt.</p>
 *
 */
public class Angebot {

	private StringBuffer raeume = new StringBuffer(),
			anfahrten = new StringBuffer();
	
	private final int kostenProKilometer;
	private int gesamtKosten = 0;
	
	/**
	 * @param kostenProKilometer Kosten pro Kilometer Anfahrtsweg.
	 */
	public Angebot(int kostenProKilometer) {
		this.kostenProKilometer = kostenProKilometer;
	}
	
	/**
	 * Kosten für die Bodenverlegung in einem einzelnen, rechteckigen Raum. Diese ergeben sich einerseits aus der Fläche des verlegten Bodens über den Preis pro Quadratmeter.
	 * Andererseits bestimmt der Umfang des Raumes die Länge der benötigten Bodenleisten, welche über den Preis pro Meter zu den Kosten beiträgt. 
	 * 
	 * @param laenge Länge des Raums
	 * @param breite Breite des Raums
	 * @param preisProQuadratmeter Kosten des gewählten Bodens (Laminat, Parkett,...) pro Quadratmeter.
	 * @param preisProMeter Kosten für die gewählte Art der Fußleiste pro Meter. Türaussparungen etc. werden vernachlässigt.
	 * 
	 * <p style="color: red;font-weight: bold;">Tip: Im Hinblick auf die Ausgabe der Angebotsdetails in {@link #getAngebotDetails()} können Sie alle benötigten Informationen in einer {@link StringBuffer} 
	 * Instanzvariablen sammeln. Sinngemäß gilt dies auch für die Gesamtkosten.</p>
	 */
	public void addiereRaum(int laenge, int breite, int preisProQuadratmeter, int preisProMeter) {
		final int kosten = laenge * breite * preisProQuadratmeter + 2 * (laenge + breite) * preisProMeter;
		raeume.append("Raum ").append(laenge).append(" x ").append(breite).append(": ").append(kosten).append("€\n");
		gesamtKosten += kosten;
	}
	
	/**
	 * Kosten für eine oder mehrere Anfahrten. Der Preis pro Kilometer wird bereits im Konstruktor {@link #Angebot(int)} festgelegt.
	 * 
	 * @param anzahl Anzahl benötigter Anfahrten
	 * @param kilometer Entfernung pro Anfahrt.
	 * 
	 * <p style="color: red;font-weight: bold;"> Tip: Im Hinblick auf die Ausgabe der Angebotsdetails in {@link #getAngebotDetails()} können Sie alle benötigten Informationen
	 * in einer (weiteren) {@link StringBuffer} Instanzvariablen sammeln. Sinngemäß gilt dies auch für die Gesamtkosten.</p>
	 */
	public void addiereAnfahrt(int anzahl, int kilometer) {
		final int kosten = anzahl * kilometer * kostenProKilometer; 
		anfahrten.append(anzahl).append(" x ").append(" Anfahrt ").append(kilometer).append(" Km: ").append(kosten).append("€\n");
		gesamtKosten += kosten;
	}
	
	/**
	 * Die Kosten ergeben sich durch alle vorherigen Aufrufe von {@link #addiereRaum(int, int, int, int)} und {@link #addiereAnfahrt(int, int)} in Verbindung
	 * mit dem im Konstruktor {@link #Angebot(int)} festgesetzten Kilometerpreis.
	 * 
	 * @return Kosten für Bodenverlegung und Anfahrt
	 */
	public int getGesamtKosten() {
		return gesamtKosten;
	}
	
	/**
	 * Die Angebotsdetails ergeben sich durch alle vorherigen Aufrufe von {@link #addiereRaum(int, int, int, int)} und {@link #addiereAnfahrt(int, int)} in Verbindung
	 * mit dem im Konstruktor {@link #Angebot(int)} festgesetzten Kilometerpreis. Die Testapplikation {@link DummyAngebot#main(String[])} erzeugt eine typische Ausgabe:
	 * 
	 * <pre><code>Raum 2 x 4: 172€
	 *Raum 3 x 5: 439€
     *3 x  Anfahrt 44 Km: 264€
     *Summe: 875€</code></pre>
	 * 
	 * @return Erläuternder Angebotstext zum Ausdrucken
	 */
	public String getAngebotDetails() {
		return raeume.toString() + anfahrten.toString() + "Summe: " + gesamtKosten + "€";
	}
}
