package de.hdm_stuttgart.mi.sd1.statistics.main;

/**
 * Validate sample input strings and convert
 * them to integer values.
 */
public class InputValidator {
  
  /**
   * Integer values being calculated upon
   * constructor call.
   */
  public final int[] values;
  
  private String errorMessage;
  private ErrorState errorState;
  
  /**
   * Transform a series of strings into integer values. In case
   * of invalid input, a corresponding error messsage 
   * and error code will be accessible via {@link #getErrorMessage()}
   * and {@link #getErrorState()} respectively.
   * 
   * Example: The array ["-1", "20", "three"]
   * contains two valid elements and the invalid element "three" which
   * cannot be converted to an integer value by virtue of
   * {@link Integer#parseInt(String)}. This will result in an
   * informative error message containing a reference to the
   * substring "three" and setting the error state to
   * {@link ErrorState#NO_INTEGER}.
   * 
   * @param userInput A set of strings possibly representing integer values.
   */
  public InputValidator(final String[] userInput) {
    
    if (0 == userInput.length) {
      values = null;
      errorState = ErrorState.NO_VALUE;
      errorMessage ="No values provided";
    } else {
      values = new int[userInput.length];
      for (int i = 0; i < userInput.length; i++) {
        try {
          values[i] = Integer.parseInt(userInput[i]);
        } catch (final NumberFormatException nfe) {
          errorState = ErrorState.NO_INTEGER;
          errorMessage = "Input string '" + userInput[i] + 
              "' does not represent an integer value";
          return;
        }
        errorState = ErrorState.OK;
        errorMessage = null;
      }
    }
  }

  /**
   * @return An Error messsage corresponding to the input string array, null
   *          if {@link #getErrorState()} equals {@link ErrorState#OK}.
   */
  public String getErrorMessage() {
    return errorMessage;
  }

  /**
   * @return Error messsage corresponding to input string array
   */
  public ErrorState getErrorState() {
    return errorState;
  }
}