package de.hdm_stuttgart.mi.sd1.stringset;

import java.util.Arrays;

/**
 * Implementing a set of strings.
 *
 */
public class MySet_String implements Set_String {

  // Indicate e.g. an object has not been found. Only index values of zero or
  // greater are to be interpreted as array index values.
  private final static int inexistentIndexValue = -1;
  
  // A set may initially harbor 10 string instances.
  private final static int initialCapacity = 10;
  
  private int capacity;
  private String [] strings;
  private int [] hashValues;
  
  private int size;
  
  private void provideRequiredCapacity() {
    if (size == capacity) {
      capacity *= 2;
      strings = Arrays.copyOf(strings, capacity);
      hashValues = Arrays.copyOf(hashValues, capacity);
    }
  }

  private int indexOf(Object obj) {
    if (obj instanceof String) {
      final int objHash = obj.hashCode();
      for (int i = 0; i < size; i++) {
        // Does an equal hash value exists? If so,
        // compare the possible candidate for object equality.
        if (objHash == hashValues[i] && strings[i].equals(obj)){
          return i;
        }
      }
    }
    return inexistentIndexValue;
  }
  
  private void init(int capacity) {
    this.capacity = capacity;
    strings = new String[initialCapacity];
    hashValues = new int[initialCapacity];
    size = 0;
  }
  
  /**
   * Constructs a new, empty set;
   */
  public MySet_String() {
    init(initialCapacity);
  }
  
  /** 
   * Copy array values into this set excluding duplicates.
   * 
   * @param source The array to copy values from
   */
  public MySet_String(final String[] source) {
    init(source.length); // May be too large in case of duplicates or null values.
    for (final String s: source) {
      add(s);
    }
  }
  
  @Override
  public int size() {
    return size;
  }

  @Override
  public boolean isEmpty() {
    return 0 == size;
  }

  @Override
  public boolean contains(Object o) {
    return inexistentIndexValue != indexOf(o);
  }

  @Override
  public String[] toArray() {
    return Arrays.copyOf(strings, size);
  }

  @Override
  public boolean add(String s) {
    if (null == s || contains(s)) {// either null or already member of set.
      return false;
    } else { // A String instance not being null and not yet part of the game.
      provideRequiredCapacity();
      strings[size] = s;
      hashValues[size++] = s.hashCode();
      return true;
    }
  }

  @Override
  public boolean remove(Object string) {
    final int index = indexOf(string);
    if (inexistentIndexValue == index) {
      return false; // String is not 
    } else {
      // Example: Set ["a", "b", "c", "d"], removing "b" with index value 1:
      //                         /    /
      //                        /    / loop index values
      //                       /    /  2 and 3
      //                      /    /
      //         ==>  ["a", "c", "d"]
      size --;
      for (int i = index; i < size; i++) {
        strings[i] = strings[i + 1];
        hashValues[i] = hashValues[i + 1];
      }
      return true;
    }
  }

  @Override
  public void clear() {
    size = 0;
  }
}
