package de.hdm_stuttgart.mi.sd1.filepath;

import java.util.StringJoiner;
import java.util.StringTokenizer;

/**
 * Represent file path data.
 *
 */
public class FileMetaInfo {
   
   static public final char fileSeparatorChar = System.getProperty("file.separator").charAt(0);

   static public final String fileSeparatorString = // Essentially the same, but we'd otherwise need tons of
         System.getProperty("file.separator");      // String.valueOf(fileSeparatorChar) calls cluttering our code
   
   public static final char noDrive = 0;
   
   static final boolean isWindowsOs = System.getProperty("os.name").startsWith("Windows"); // true for Windows ... XP / 7 / 8 / 10 / ...
   
   // Drive letter like 'C:' on windows. Value will be set to
   // class variable noDrive's value
   // for relative path specifications or
   // on non-windows platforms.
   public final char drive;
      
   public final String path, basename, extension;

   // Paths may be either relative or absolute.
   public final boolean isRelative;
   
   /**
    * Starting from a string representation of a filename.
    * 
    * @param filePathName e.g. "C:\programms\openvpn\bin\openvpn.exe"
    */
   public FileMetaInfo (final String filePathName) {
      
      final StringTokenizer pathComponents;
      
      if (isWindowsOs) {
         if (filePathName.charAt(1) == ':') { // Absolute path starting with a drive letter like e.g. 'C:'
            isRelative = false;
            drive = filePathName.charAt(0);
            pathComponents = new StringTokenizer(filePathName.substring(2), fileSeparatorString); // Ignoring first two drive related letters
         } else { // relative path not starting with a drive letter
            isRelative = true;
            drive = noDrive;
            pathComponents = new StringTokenizer(filePathName, fileSeparatorString);
         }
      } else {
         drive = noDrive;
         if (filePathName.charAt(0) == fileSeparatorChar) {
            isRelative = false;
         } else {
            isRelative = true;
         }
         pathComponents = new StringTokenizer(filePathName, fileSeparatorString);
      }
      // Now we must scan for directory separators (either '/' on UNIX or '\' on Windows)

      final StringJoiner directoryComponents = new StringJoiner(fileSeparatorString);
      String currentToken;
      do {
         currentToken = pathComponents.nextToken();// May be either a directory or a filename.
         if (pathComponents.hasMoreElements()) { // There is more to come, curentToken still is a directory
            directoryComponents.add(currentToken);
         }
      } while (pathComponents.hasMoreElements());
      
      if (isRelative) {
         path = directoryComponents.toString();
      } else { // Absolute paths start by either '/' or '\' 
         path = String.valueOf(fileSeparatorChar).concat(directoryComponents.toString());
      }
      
      final String filename = currentToken; // Not strictly required, for documentation's sake only
         
      // Now (if possible) decompose filename xyz.ext into its basename "xyz" and extension "ext"
      
      final int indexOfPointCharacter = filename.indexOf('.');
      
      if (-1 == indexOfPointCharacter) { // Filename does not contain any '.' character
         basename = filename;
         extension = null;
      } else {
         basename = filename.substring(0, indexOfPointCharacter);
         extension = filename.substring(indexOfPointCharacter + 1);
      }
   }

   @Override
   public String toString() {
      
      final StringBuffer filename = new StringBuffer();
      if (isWindowsOs && !isRelative) {
         filename.append(drive);
      }
      return filename.append(path).append(fileSeparatorChar).append(basename).append('.').append(extension).toString();
   }
}