package de.hdm_stuttgart.mi.sd1.marks;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Creating mark statistics.
 *
 */
public class TextFileHandler {
  
  /**
   * The input file name to read from.
   */
  public final String filename;
  
  /**
   * The list of marks found so far among with their frequencies.
   */
  final Map<Character, Integer> frequencyByMark = new Hashtable<Character, Integer>();
  
  /**
   * @param filename The filename input to read input, see {@link #filename}
   * 
   * @throws IOException Input file cannot be read
   * @throws FileNotFoundException File does not exist
   */
  public TextFileHandler(final String filename)
      throws FileNotFoundException, IOException {
    
    this.filename = filename;
    final BufferedReader reader = new BufferedReader(new FileReader(filename));
    for (String line = reader.readLine(); line != null; line = reader.readLine()) {
      // Ignore white space and some common text markers.
      final String[] words = line.trim().split("[ \t]*,[ \t]*");
      
      if (2 == words.length ) {// Line has got three comma separated components
        final String markString = words[1].trim();
        switch (markString) {// Third component really corresponds to a valid mark
        case "A":
        case "B":
        case "C":
        case "D":
          final Character mark = markString.charAt(0);
          final Integer currentFrequency = frequencyByMark.get(mark); 
          if (null == currentFrequency) { // No such mark found yet
            frequencyByMark.put(mark, new Integer(1));
          } else {// Mark already exists, increment frequency
            frequencyByMark.put(mark, new Integer(currentFrequency + 1));
          }
          break;
          default:
            System.err.println("Ignoring erroneous mark '" + markString + "' in '" + line + "'");
        }
        
      } else { // Fewer or more than two components
        System.err.println("Ignoring erroneous line '" + line + "'");
      }
    }
    reader.close();
  }
  
  /**
   * Print sorted list of marks along with corresponding frequencies
   */
  public void printMarkFrequencies() {
    
    final List<Entry<Character, Integer>> markList = new ArrayList<>(frequencyByMark.entrySet());
    
    Collections.sort(markList, new MarkComparator());
    for (final Entry<Character, Integer> entry : markList) {
      System.out.format("%s: %d\n",
                         entry.getKey(),
                              entry.getValue());
    }
  }
  
  static class MarkComparator implements Comparator<Entry<Character, Integer>> {

    @Override
    public int compare(Entry<Character, Integer> e1,
        Entry<Character, Integer> e2) {
      return e1.getKey().compareTo(e2.getKey());
    }
    
  }
}
