package de.hdm_stuttgart.mi.sd1.marks;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Getting mark statistics.
 *
 */
public class TextFileHandler {
  
  /**
   * The input file name to read from.
   */
  public final String filename;
  
  /**
   * The marks found so far among with their name lists.
   */
  final Map<Character, List<String>> frequencyByMark = new Hashtable<Character, List<String>>();
  
  /**
   * @param filename The filename input to read input, see {@link #filename}
   * 
   * @throws IOException Input file cannot be read
   * @throws FileNotFoundException File does not exist
   */
  public TextFileHandler(final String filename)
      throws FileNotFoundException, IOException {
    
    this.filename = filename;
    final BufferedReader reader = new BufferedReader(new FileReader(filename));
    for (String line = reader.readLine(); line != null; line = reader.readLine()) {
      // Ignore white space and some common text markers.
      final String[] words = line.trim().split("[ \t]*,[ \t]*");
      
      if (2 == words.length ) {// Line has got three comma separated components
        final String markString = words[1].trim();
        switch (markString) {// Third component really corresponds to a valid mark
        case "A":
        case "B":
        case "C":
        case "D":
          final String name = words[0];
          final Character mark = markString.charAt(0);
          List<String> currentFrequency = frequencyByMark.get(mark); 
          if (null == currentFrequency) { // No such mark found yet
            currentFrequency = new ArrayList<String>();
            frequencyByMark.put(mark, currentFrequency);
          } 
          currentFrequency.add(name);// Add new name to list
          break;
          default:
            System.err.println("Ignoring erroneous mark '" + markString + "' in '" + line + "'");
        }
      } else {
        System.err.println("Ignoring erroneous line '" + line + "'");
      }
    }
    reader.close();
  }
  
  /**
   * Print sorted list of marks along with corresponding frequencies
   */
  public void printMarkFrequencies() {
    
    final List<Entry<Character, List<String>>> markList = new ArrayList<>(frequencyByMark.entrySet());
    
    Collections.sort(markList, new MarkComparator());
    for (final Entry<Character, List<String>> entry : markList) {
      System.out.print(entry.getKey() + ": ");
      printList(entry.getValue());
      System.out.println();
    }
  }
  
  static void printList(final List<String> names) {
    Collections.sort(names);
    String separator = "";
    for (final String name: names) {
      System.out.print(separator + name);
      separator = ", ";
    }
  }
  
  static class MarkComparator implements Comparator<Entry<Character, List<String>>> {

    @Override
    public int compare(Entry<Character,  List<String>> e1,
        Entry<Character,  List<String>> e2) {
      return e1.getKey().compareTo(e2.getKey());
    }
  }
}