package de.hdm_stuttgart.mi.sd1.nonsenseGenerator;

import de.hdm_stuttgart.mi.nonsense.SequenceGenerator;

/**
 * Mocking a random generator for test purposes. Instances cycle through predefined
 * values allowing for tests of random processes.
 * 
 * Outside test scope a "real" random number generator has to be used.
 *
 */
public class PredictableSequenceGenerator implements SequenceGenerator {
   
   final int [] randomCycleValues; // initial value
   int index = 0;

   /**
    * These values will be cyclically returned by {@link #getNextValue(int)}. Example:
    * 
    *  Setting to [0,3,1] {@link #getNextValue(int)} will return 0,3,1,0,3,1,... provided
    *  its argument bound is at least 4.
    *  
    *  Array values exceeding {@link #getNextValue(int)}'s bound argument will be corrected to (bound-1), negative values
    *  will be corrected to 0. Thus having bound == 2 in place the array [-5,3]
    *   {@link #getNextValue(int)} will yield cyclic values 0,1,0,1,...
    * 
    * @param randomCycleValues These values define the cycle of "random" values to be returned
    * by {@link #getNextValue(int)} in a completely deterministic fashion.
    */
   public PredictableSequenceGenerator(final int[] randomCycleValues) {
      this.randomCycleValues = randomCycleValues;
   }
   
   @Override
   public int getNextValue(int bound) {
      if (randomCycleValues.length <= index ) {
         // We've reached the upper limit: Restart cycle.
         index = 0;
      }
      final int ret = Math.max(0, Math.min(randomCycleValues[index], bound - 1));
      index++;
      return ret;
   }
}
