package de.hdm_stuttgart.mi.sd1.wc;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;

/**
 * Count lines, words and characters for an individual
 * file name.
 *
 */
public class TextFileStatistics {

  private static int // In case of multiple input files we need to calculate totals
      totalNumLines = 0,
      totalNumWords = 0, 
      totalNumCharacters = 0;
  
  /**
   * The name of the file content had been read, empty string if content
   * was read from standard input.
   */
  public final String filename;
  
  /**
   * @return The total number of lines from all files being read so far.
   */
  public static int getTotalNumLines() {
    return totalNumLines;
  }

  /**
   * @return The total number of words from all files being read so far.
   */
  public static int getTotalNumWords() {
    return totalNumWords;
  }

  /**
   * @return The total number of characters from all files being read so far.
   */
  public static int getTotalNumCharacters() {
    return totalNumCharacters;
  }

  /**
   * The current text file's number of lines
   */
  public final int numLines;
      
  /**
   * The current text file's number of words
   */
  public final int numWords;
  
  /**
   * The current text file's number of characters
   */
  public final int numCharacters;
  
  /**
   * @param reader Reader supplying content to be read and parsed.
   * @param filename The filename input has been read from, empty string
   *        if reading from standard input, see {@link #filename}
   * 
   * @throws IOException File fileName cannot be read
   * @throws FileNotFoundException File fileName does not exist
   */
  public TextFileStatistics(final BufferedReader reader, final String filename)
      throws FileNotFoundException, IOException {
    
    this.filename = filename;

    int tmpNumLines = 0,        // We need local variables to sum up and assign the results 
        tmpNumWords = 0,        // to their final (immutable) counterparts numLines,
        tmpNumCharacters = 0;   // numWords and numCharacters.
    
    for (String line = reader.readLine(); line != null; line = reader.readLine()) {
      tmpNumLines++;
      tmpNumWords += findNoOfWords(line); 
      tmpNumCharacters += line.length() + 1; // +1 representing the chopped newline character
    }
    
    numLines = tmpNumLines;           // Now assign the immutable
    numWords = tmpNumWords;           // instance attributes.
    numCharacters = tmpNumCharacters;
    
    totalNumLines += numLines;        // We need these in case of multiple input
    totalNumWords += numWords;        // files for presenting the grand total.
    totalNumCharacters += numCharacters;
  }
  
  /**
   * Count the number of words in a string. Any sequence of non-whitespace
   * characters (space, \t and \n) is being considered a word.
   * 
   * @param line The input string to be examined
   * @return The number of words in the given String.
   */
  public static int findNoOfWords (final String line) {
    int noOfWords = 0;
    boolean isWhitespace = true;
    for (final char ch: line.toCharArray()) {
      switch (ch) {
      case ' ':  // Whitespace
      case '\t': // Whitespace
      case '\n': // Whitespace
        isWhitespace = true;
        break;
      
      default:  // Non-whitespace
        if (isWhitespace) {
          isWhitespace = false;
          noOfWords++;
        }
      }
    }
    return noOfWords;
  }
}