package de.hdm_stuttgart.sda1.insert;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLIntegrityConstraintViolationException;

import java.util.ResourceBundle;
import java.util.Scanner;

/**
 * <p>A JDBC "getting started" example inserting person's data into a table</p>
 *
 * <p>Caveat: No error handling whatsoever is being supplied!</p>
 *
 * <p>Preconditions:</p>
 *
 * <ul>
 *   <li>A database matching {@link #main(String[])}'s connection setup must be configured.</li>
 *   <li>A database table »Person« corresponding to this project's file »resources/schema.sql« must exist.</li>
 * </ul>
 *
 */
public class PreparedInsert {

  static private final Logger log = LogManager.getLogger (PreparedInsert.class);

  static final ResourceBundle jdbcProperties = ResourceBundle.getBundle("jdbc");

  static final  String insertUserStatement = "INSERT INTO Person VALUES(?, ?)";

  /**
   * Asking for user input inserting related person records into database.
   *
   * @param args unused
   *
   */
  public static void main(String[] args) {

    try (final Connection conn = DriverManager.getConnection(
      jdbcProperties.getString("jdbcurl"),
      jdbcProperties.getString("username"),
      jdbcProperties.getString("password"))) {
      handleUserInput(conn.prepareStatement(insertUserStatement));
    } catch (final SQLException e) {
      log.error("General database connection problem:", e);
    }
  }
  static private void handleUserInput(final PreparedStatement statement) throws SQLException {
    try (final Scanner scanner = new Scanner(System.in)) {

      while(true) {
        System.out.print("Enter a person's name or 'x' to exit: ");
        final String name = scanner.nextLine().trim();
        if (name.equals("x")) {
          break;
        }
        System.out.print("Enter " + name + "'s email or 'x' to exit: ");
        final String email = scanner.nextLine().trim();
        if (email.equals("x")) {
          break;
        }
        final int insertCount = insertPerson(statement, name, email);
        if (1 == insertCount) {
          System.out.println("Successfully inserted new user '" + name + "'\n");
        } else {
          System.out.println("Insertion failed, duplicate email '" + email + "' ?\n");
        }
      }
      System.out.println("Bye!");
    }
  }
  /**
   * <p>Try inserting new Person record.</p>
   *
   * @param statement To be used for SQL INSERT attempt.
   * @param name Person's name
   * @param email Person's email
   * @return Inserted reord count: 1 on successful INSERT, 0 in case of duplicate email violating UNIQUE constraint.
   *
   * @throws SQLException To be thrown in case of non - {@link java.sql.SQLIntegrityConstraintViolationException}
   * errors.
   */
  static public int insertPerson(final PreparedStatement statement, final String name, final String email)
    throws SQLException {
    statement.setString(1, name);
    statement.setString(2, email);
    log.info("Executing '" + insertUserStatement + "' using " + name + " and " + email);

    try {
      return statement.executeUpdate();
    } catch (final SQLIntegrityConstraintViolationException ex) {
      log.info("Constraint violation, possibly inserting duplicate email '" + email + "'");
      return 0;
    }
  }
}