package de.hdm_stuttgart.mi.persistence;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import de.hdm_stuttgart.mi.sda1.hashprovider.HashProvider;

/**
 * Handle database communication internally using {@link PreparedStatement} instances
 * avoiding SQL injection attacks. There are two
 * distinct internal states <strong>disconnected</strong> and <strong>connected</strong>, see
 * {@link #isConnected()}. State transitions happen on invocation of
 * {@link #connect(String, String)} or {@link #disconnect()}.
 * 
 * The following snippet illustrates the intended usage:
 * <pre>   public static void main(String[] args) {
      final PersistenceHandler ph = new PersistenceHandler();
      if (ph.connect()) {
         if (!ph.add("Jim", "jim@foo.com")) {
            System.err.println("Insert Error:" + ph.getErrorMessage());
         }
      } else {
         System.err.println("Connect error:" + ph.getErrorMessage());
      }
   }</pre>
 * 
 */
public class PersistenceHandler {

	/**
	 * GUI messages
	 *
	 */
	static public enum Msg {

		/**
		 * Connection established
		 */
		CONNECT_SUCCESSFUL("Connect successful"),

		/**
		 * Connection established
		 */
		DISCONNECT_SUCCESSFUL("Disconnect successful"),

		/**
		 * Connection to database server fails
		 */
		UNABLE_TO_CONNECT("Unable to connect"),

		/**
		 * Possible user credential problem
		 */
		UNABLE_TO_AUTHENTICATE("Unable to authenticate"),

		/**
		 * Possible JDBC driver unavailability.
		 */
		JDBC_DRIVER_FAILURE("Your database driver could not be loaded!"),

		;

		private Msg(final String msg) {
			this.msg = msg;
		}

		/**
		 * Last operation / current state description
		 */
		public final String msg;

		@Override
		public String toString() {
			return msg;
		}
	}


	static {

		boolean sqlDriverRegistryCompletedTmp = false;
		final String sqlDriverClassName = DbProps.getString("PersistenceHandler.driverClassName");
		final Logger logTmp = LogManager.getLogger(PersistenceHandler.class);
		log = logTmp;
		try {
			Class.forName(sqlDriverClassName);
			sqlDriverRegistryCompletedTmp = true;
		} catch (final ClassNotFoundException e) {
			logTmp.error("Unable to register SQL driver class '" + sqlDriverClassName + "'", e);
		}
		sqlDriverRegistryCompleted = sqlDriverRegistryCompletedTmp;
	}

	private final static Logger log;
	static final boolean sqlDriverRegistryCompleted;

	final HashProvider hashProvider = new HashProvider(4);

	Connection conn = null;
	PreparedStatement stmt = null;

	final PersistView loginView;

	/**
	 * New instances are in <strong>disconnected</strong> state. See {@link #isConnected()}
	 * @param loginView Forward both informative and error messages plus status updates.
	 */
	public PersistenceHandler(final PersistView loginView) {
		this.loginView = loginView;
	}

	/**
	 * Inserting a (name, email) record into the database server. Errors / informational messages
	 * status updates are being logged and forwarded via {@link PersistView}.
	 * 
	 * <dl>
	 *   <dt><b>Precondition:</b></dt>
	 *   <dd>must be in <strong>connected</strong> state, see {@link #connect(String, String)}</dd>
	 * </dl>
	 * 
	 * @param name A person's name
	 * @param email A person's email address
	 * @return true on successful INSERT, false otherwise.
	 * 
	 */
	public boolean insert(final String name, final String email, final String login, final String password) {
		try {
			stmt.setString(1, name);
			stmt.setString(2, email);
			stmt.setString(3, login);
			stmt.setString(4, hashProvider.getSaltedHash(password));

			if (1 == stmt.executeUpdate()) {
				loginView.guiInfo("New user '" + name + "' has been successfully inserted.");
				loginView.setDatasetInserted();
				return true;
			} else {
				loginView.guiError("Unknown error inserting user '" + name + "'");
			}
		} catch (final SQLException e) {
			final String sqlState = e.getSQLState();
			if (sqlState.startsWith("23")) {
				loginView.guiError("E-mail already in database");
			} else {
				loginView.guiError("Unknown error, see log file");
			}
			log.error("Unable to insert user: '" + e.getMessage() + "', SQLState=" + sqlState);
		}
		return false;
	}

	/**
	 * @return connected to database server?
	 */
	public boolean isConnected() {
		return null != conn;
	}

	/**
	 * Connect to database server and authenticate user.
	 * 
	 * @param login The user's login name
	 * @param password The user's clear text password.
	 */
	public void connect(final String login, final String password) {

		if (!sqlDriverRegistryCompleted) {
			loginView.guiError(Msg.JDBC_DRIVER_FAILURE.msg);
		} else {
			try {
				log.info("Connecting to database ...");
				conn = DriverManager.getConnection(
						DbProps.getString("PersistenceHandler.jdbcUrl"),
						DbProps.getString("PersistenceHandler.username"),
						DbProps.getString("PersistenceHandler.password"));
				if (!authenticate(login, password)) {
					conn.close();
					conn = null;
				}
			} catch (final SQLException e) {
				log.error("Unable to open connection:", e);
				loginView.guiError(Msg.UNABLE_TO_CONNECT.msg);
				conn = null;
			}
		}
	}

	boolean authenticate(final String login, final String password) {

		// Using a PreparedStatement in favor of an ordinary Statement
		// prohibiting SQL injection attacks.
		//
		loginView.setStatusDisconnected(); // start from scratch
		try (final PreparedStatement getHash =
				conn.prepareStatement("SELECT password FROM Person WHERE login = ?")) {
			getHash.setString(1, login);

			try (final ResultSet loginRecord = getHash.executeQuery()) {
				if (loginRecord.next()) {
					final String passwordHash = loginRecord.getString("password");
					if (HashProvider.check(password.toCharArray(), passwordHash)) {
						stmt = conn.prepareStatement("INSERT INTO Person VALUES(?, ?, ?, ?)");
						loginView.guiInfo(Msg.CONNECT_SUCCESSFUL.msg);
						loginView.setStatusConnected();
						return true;
					} 
				}
			}
		} catch (final SQLException e) {
			log.fatal("Failed reading credentials", e);
		}
		loginView.guiError("Unable to authenticate");
		return false;
	}

	/**
	 * Disconnect from database server.
	 */
	public void disconnect() {
		if (isConnected()) {
			try {
				log.info("Trying to disconnect");
				stmt.close();
				conn.close();
				loginView.guiInfo(Msg.DISCONNECT_SUCCESSFUL.msg);
				loginView.setStatusDisconnected();
			} catch (SQLException e) {
				log.error("Unable to close connection:\"" + e.getMessage() + "\".");
				loginView.guiError("Disconnect error, see log for details");
			}
			conn = null;
		}
	}
}