package de.hdm_stuttgart.mi.persistence;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Handle database communication. There are two
 * distinct internal states <strong>disconnected</strong> and <strong>connected</strong>, see
 * {@link #isConnected()}. These two states may be toggled by invoking
 * {@link #toggleConnectionState()}.
 * 
 * The following snippet illustrates the intended usage:
 * <pre>   public static void main(String[] args) {
      final PersistenceHandler ph = new PersistenceHandler();
      if (ph.connect()) {
         if (!ph.add("Jim", "jim@foo.com")) {
            System.err.println("Insert Error:" + ph.getErrorMessage());
         }
      } else {
         System.err.println("Connect error:" + ph.getErrorMessage());
      }
   }</pre>
 * 
 * @author goik
 */
public class PersistenceHandler {
   
   static {
      
      boolean sqlDriverRegistryCompletedTmp = false;
      final String sqlDriverClassName = DbProps.getString("PersistenceHandler.driverClassName");
      final Logger logTmp = LogManager.getLogger(PersistenceHandler.class);
      log = logTmp;
      try {
         Class.forName(sqlDriverClassName);
         sqlDriverRegistryCompletedTmp = true;
      } catch (ClassNotFoundException e) {
         logTmp.error("Unable to register SQL driver class '" + sqlDriverClassName + "'");
      }
      sqlDriverRegistryCompleted = sqlDriverRegistryCompletedTmp;
   }

   private final static Logger log;
   static final boolean sqlDriverRegistryCompleted;

   Connection conn = null;
   Statement stmt = null;
   
   final PersistView loginView;

   /**
    * New instances are in <strong>disconnected</strong> state. See {@link #isConnected()}
    * @param loginView Forward both informative and error messages plus status updates.
    */
   public PersistenceHandler(final PersistView loginView) {
      this.loginView = loginView;
   }

   /**
    * Inserting a (name, email) record into the database server. Errors / informational messages
    * status updates are being logged and forwarded via {@link PersistView}.
    * 
    * <dl>
    *   <dt><b>Precondition:</b></dt>
    *   <dd>must be in <strong>connected</strong> state, see {@link #toggleConnectionState()}</dd>
    * </dl>
    * 
    * @param name A person's name
    * @param email A person's email address
    * @return true on successful INSERT, false otherwise.
    * 
    */
   public boolean insert(final String name, final String email) {
      final String sql = "INSERT INTO Person VALUES('" + name + "', '" + 
            email + "')"; 
      try {
         final int insertCount = stmt.executeUpdate(sql);
         loginView.guiInfo("New user '" + name + "' has been successfully inserted.");
         loginView.setDatasetInserted();
         return 1 == insertCount;
      } catch (SQLException e) {
         final String sqlState = e.getSQLState();
         if (sqlState.startsWith("23")) {
            loginView.guiError("E-mail already in database");
         } else {
            loginView.guiError("Unknown error, see log file");
         }
         log.error("Unable to execute '" + sql + "': '" + e.getMessage() + "', SQLState=" + sqlState);
      }
      return false;
   }

   /**
    * @return connected to database server?
    */
   public boolean isConnected() {
      return null != conn;
   }
   /**
    * Switch between connected and disconnected state.
    */
   public void toggleConnectionState() {
      
      if (!sqlDriverRegistryCompleted) {
         loginView.guiError("Your database driver could not be loaded! " +
               "This application won't work, see log file for details.");
         return;
      }

      if (null == conn) { // Disconnected --> connect
         try {
            log.info("Trying to connect");
            conn = DriverManager.getConnection(
                  DbProps.getString("PersistenceHandler.jdbcUrl"),
                  DbProps.getString("PersistenceHandler.username"),
                  DbProps.getString("PersistenceHandler.password"));
            try {
               stmt = conn.createStatement();
               loginView.guiInfo("Connection to database established.");
               loginView.setStatusConnected(); // Notify GUI
               return;
            } catch (SQLException e) {
               log.error("Connection opened but Statement creation failed:\"" + e.getMessage() + "\"."); 
               try {
                  conn.close();
               } catch (SQLException ee) {
                  log.error("Closing connection failed:\"" + e.getMessage() + "\".");
               }
               conn = null;
            }
         } catch (SQLException e) {
            log.error("Unable to open connection:\"" + e.getMessage() + "\".");
         }
         loginView.guiError("Connect error, see log for details");

      } else { // Connected --> disconnect

         try {
            log.info("Trying to disconnect");
            conn.close();
            loginView.guiInfo("Connection to database closed.");
            loginView.setStatusDisconnected();
         } catch (SQLException e) {
            log.error("Unable to close connection:\"" + e.getMessage() + "\".");
            loginView.guiError("Disconnect error, see log for details");
         }
         conn = null;
      }
   }
}