package de.hdm_stuttgart.mi.javastreams;

import static org.hamcrest.MatcherAssert.assertThat;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.OptionalDouble;
import java.util.stream.Collector;
import java.util.stream.Collectors;

import org.hamcrest.Matchers;
import org.junit.Assert;
import org.junit.Test;

import com.google.common.collect.ImmutableList;
import com.google.common.collect.ImmutableMap;

import de.hdm_stuttgart.mi.javastreams.Student.Sex;

/**
 * Testing functional queries.
 */
public class Java8FunctionalTest {

   final List<Student> roster = Student.createRoster();

   /**
    * Order all male students by email and create a list of their respective names
    * in that order eliminating possible duplicates:
    * 
    *  "Bob", 2, Student.Sex.MALE, "bob@uk.edu"
    *  "Fred", 2, Student.Sex.MALE, "fred@example.com"
    *  "George", 4, Student.Sex.MALE, "george@math.edu"
    *  "Jane", 1, Student.Sex.FEMALE, "jane@kiv.de"
    *  "Kim", 2, Student.Sex.FEMALE, "wilde@serious.de"
    *  
    *  ==> {"Bob", "Fred", "George"}
    *  
    */
   @Test
   public void allMaleDistinctNameOrderedByEmail() {

      final List<String> emails =
            roster.parallelStream().
            filter(s -> s.gender == Sex.MALE).
            sorted((s1, s2) -> s1.getEmailAddress().compareTo(s2.getEmailAddress())).
            map(Student::getName).
            distinct().
            collect(Collectors.toList());

      assertThat(
            emails,
            Matchers.<List<String>> equalTo( 
                  ImmutableList.of("Bob", "Fred", "George") 
                  ) 
            );
   }

   /**
    * Summing students' marks having a German email address:
    * 
    * "Jane", 1, Student.Sex.FEMALE, "jane@kiv.de"
    * "Kim",  2, Student.Sex.FEMALE, "wilde@serious.de"
    * 
    */
   @Test
   public void markSumAllStudentsMarksHavingEmail_Dot_de() {
      Assert.assertEquals(
            3,                //Expected marks sum 3 == 1 + 2   
            roster.parallelStream()
            .filter(p -> p.emailAddress.endsWith(".de")) // Jane and Kim
            .mapToInt(Student::getMark)
            .reduce(0, (a, b) -> a + b)
            );
   }

   /**
    * A comma separated string containing all students' alphabetically ordered
    * email addresses:
    * 
    * "bob@uk.edu, fred@example.com, george@math.edu, ..."
    *  
    */
   @Test
   public void orderedCharacterSeparatedEmailList() {

      Assert.assertEquals(
            "bob@uk.edu, fred@example.com, george@math.edu, jane@kiv.de, wilde@serious.de", 

            roster.parallelStream().
            map(Student::getEmailAddress).
            sorted().
            collect(new CsvCollector(", ")) // Using ", " as separator string.
            );

   }

   /**
    * Achieves the same result as in {@link #orderedCharacterSeparatedEmailList()}
    * internally using a standard collector with a modified
    * {@link Collector#finisher()} method:
    * 
    * "bob@uk.edu, fred@example.com, george@math.edu, ..."
    *  
    */
   @Test
   public void orderedCharacterSeparatedEmailList2() {

      Assert.assertEquals(
            "bob@uk.edu, fred@example.com, george@math.edu, jane@kiv.de, wilde@serious.de", 

            roster.parallelStream().
            map(Student::getEmailAddress).
            sorted().
            collect
            (Collectors.collectingAndThen
                  (Collectors.toList(),
                        l -> l.stream().   // A stream within a stream.
                        reduce((s1, s2) -> s1 + ", " + s2).
                        get()
                        )
                  )
            );
   }

   /**
    * Get the average mark of all female students as double value.
    * 
    * Female marks: {1, 2}, sum == 3, average = 3. / 2 == 1.5 
    * 
    */
   @Test
   public void averageMarkFemaleStudents() {

      final OptionalDouble femaleAverage =
            roster.parallelStream().
            filter(s -> s.gender == Sex.FEMALE).
            mapToInt(Student::getMark).
            average();

      Assert.assertTrue(femaleAverage.isPresent());
      Assert.assertEquals((1 + 2) / 2., femaleAverage.getAsDouble(), 1.E-20);

   }

   /**
    * Group students by sex and collect their respective names:
    * 
    * Female: 
    *     "Jane"
    *     "Kim"
    *
    * Male:
    *     "Fred"
    *     "George"
    *     "Bob"
    * 
    */
   @Test
   public void studentNamesBySex() {

      final Map<Student.Sex, List<String>> studentnamesBySex =
            roster
            .parallelStream()
            .collect(
                  Collectors.groupingBy(
                        Student::getSex,
                        Collectors.mapping(
                              Student::getName,
                              Collectors.toList())
                        )
                  );

      assertThat(
            studentnamesBySex,
            Matchers.<Map<Student.Sex, List<String>>> equalTo( 
                  ImmutableMap.of (
                        Student.Sex.MALE,   ImmutableList.of("Fred", "George", "Bob"),
                        Student.Sex.FEMALE, ImmutableList.of("Jane", "Kim")
                        ) 
                  ) 
            );
   }

   /**
    * Marking frequencies:
    * 
    * Mark 1: "Jane"               --- 1 student
    * Mark 2: "Fred", "Bob", "Kim" --> 3 students
    * Mark 4: "George"             --> 1 student
    * 
    */
   @Test
   public void markingFrequencies() {
      final Map<Integer, Integer> frequencyByMark =
            roster
            .parallelStream()
            .collect(
                  Collectors.groupingBy(
                        Student::getMark,
                        Collectors.reducing(
                              0,
                              s -> 1,// counting heads.
                              Integer::sum)
                        )
                  );

      assertThat(frequencyByMark,
            Matchers.<Map<Integer, Integer>> equalTo( 
                  ImmutableMap.of(
                        1,   1,
                        2,   3,
                        4,   1
                        ) 
                  ) 
            );
   }

   /**
    * Students' names in ascending order grouped by by marking category:
    * 
    * Mark 1: {"Jane"}
    * Mark 2: {"Bob", "Fred", "Kim"}
    * Mark 4: {"George"} 
    *
    */
   @Test
   public void studentnamessByMark() {

      final Map<Integer, List<String>> namesByMark =
            roster
            .parallelStream()
            .collect(
                  Collectors.groupingBy(
                        Student::getMark,
                        Collectors.mapping(
                              Student::getName,
                              Collectors.collectingAndThen(
                                    Collectors.toList(),
                                    l -> {
                                       Collections.sort(l);
                                       return l;
                                    }
                                    )
                              )
                        )
                  );

      assertThat(
            namesByMark,
            Matchers.<Map<Integer, List<String>>> equalTo ( 
                  ImmutableMap.of(
                        1,   ImmutableList.of("Jane"),
                        2,   ImmutableList.of("Bob", "Fred", "Kim"),
                        4,   ImmutableList.of("George")
                        ) 
                  ) 
            );
   }
}
